<?xml version="1.0" encoding="UTF-8"?>
<con:xqueryEntry xmlns:con="http://www.bea.com/wli/sb/resources/config">
  <con:xquery>declare namespace autn = "http://schemas.autonomy.com/aci/";

declare variable $query as xs:string external;
declare variable $body as element()* external;

(: Returns first matched query term from a query string :)
declare function local:next-term($query as xs:string?)  as xs:string? {
	if ($query)
		then fn:replace($query, '^[\s]*(("[^"]*")|([\w\u0080-\uFFFF]+\:\_\*\?\/)|([^\s]+)).*$', "$1")
		else ()
};

(: Removes first matched query term from the query string :)
declare function local:remove-term($query as xs:string?)  as xs:string? {
	if ($query) 
		then	fn:substring-after($query, local:next-term($query))
		else ()
};

(: Create map of terms and their synonyms out of result set :)
declare function local:get-synonym-terms($list as element(autn:hit)*) as element(term)* {
	for $element in $list
	return 
		for $link in fn:lower-case(fn:normalize-space(local:next-term($element/autn:content/DOCUMENT/DRECONTENT/text())))
		return 
			if (fn:normalize-space($link) = '') 
			then ()
			else
				&lt;term>
					&lt;link>{$link}&lt;/link>
					&lt;synonyms>{
						fn:replace(
							fn:lower-case(fn:normalize-space($element /autn:content/DOCUMENT/DRECONTENT/text())), 
							fn:concat($link, "(?=[\s$])"), 
							""
						)
					}&lt;/synonyms>
				&lt;/term>	
};

(: Create replacement for the given term based on the map of known synonym terms :)
declare function local:term-replace($term as xs:string?, $list as element(term)*)  as xs:string {
	if ($term)
		then
			fn:concat(
				$term, 
				if (fn:not(fn:empty($list/link[text() = fn:lower-case(fn:normalize-space($term))]))) 
					then fn:concat(" (", $list/link[text() = fn:lower-case(fn:normalize-space($term))]/../synonyms/text(), ") ")
					else xs:string("")
			)
		else xs:string("")
};

(: Append synonym terms to the given query string based on the synonym term list :)
declare function local:query-replace($query as xs:string?, $list as element(autn:hit)*)  as xs:string {
	if (not($query) or (fn:normalize-space($query) = ""))
		then xs:string("")
		else fn:concat(
					local:term-replace(
						local:next-term($query),
						local:get-synonym-terms($list)
					),
					xs:string(" "),
					local:query-replace(local:remove-term($query), $list)
				)
};

&lt;query>{fn:normalize-space(local:query-replace($query, $body//autn:hit))}&lt;/query></con:xquery>
</con:xqueryEntry>